@@ -92,7 +92,6 @@ def load_lsf(lsf_fname, wl, nsub=1):
 
     nsub : integer  [default = 1]
         Kernel subsampling factor relative to the data.
-        This is only used if the resolution is given as a LSF file.
 
     Returns
     -------
@@ -116,6 +115,9 @@ def load_lsf(lsf_fname, wl, nsub=1):
 
     # The LSF data is the resulting table excluding the first line:
     lsf = lsf_tab[1:, :]
+    # Normalize the LSF:
+    lsf_norm = np.sum(lsf, axis=0)
+    lsf = lsf/lsf_norm
 
     # Make an array of pixel indeces:
     lsf_pix = np.arange(lsf.shape[0])
@@ -137,8 +139,8 @@ class Region():
 
         .. rubric:: Attributes
 
-        velspan : float
-            The velocity range to used for the fitting region.
+        velspan : Tuple(float, float)
+            The velocity ranges used for the fitting region. Given as a range by a tuple of (lower, upper).
 
         lines : list(:class:`dataset.Line`)
             A list of Lines defined in the region.
@@ -178,6 +180,13 @@ class Region():
             A spectral identifier to point back to the raw data chunk.
 
         """
+        if hasattr(velspan, '__iter__'):
+            if len(velspan) == 2:
+                pass
+            else:
+                raise ValueError("argument 'velspan' must have two values! not %i" % len(velspan))
+        else:
+            velspan = (-1.*np.abs(velspan), np.abs(velspan))
         self.velspan = velspan
         self.specID = specID
         if line:
