@@ -12,6 +12,7 @@ from scipy.signal import fftconvolve, gaussian
 from scipy.ndimage import gaussian_filter1d
 from scipy.interpolate import RectBivariateSpline as spline2d
 import numpy as np
+import re
 
 from ..utils import Asplund
 from ..utils import molecules
@@ -19,6 +20,7 @@ from ..container.lines import Line
 from ..funcs.voigt import evaluate_profile
 from ..utils import terminal_attributes as term
 from ..funcs import voigt
+from ..container.regions import load_lsf
 
 plt.rcParams['lines.linewidth'] = 1.0
 plt.rcParams['font.family'] = 'Arial'
@@ -341,25 +343,28 @@ def plot_all_lines(dataset, plot_fit=True, rebin=1, fontsize=12, xmin=None,
                     delta_lam = (l0*(dataset.redshift + 1) - l_ref)
                     delta_v = delta_lam / l_ref * 299792.458
 
+                    if hasattr(region.velspan, '__iter__'):
+                        vmin, vmax = region.velspan
+                    else:
+                        vspan = region.velspan
+                        vmin = -vspan
+                        vmax = vspan
 
                     if xmin:
                         if xunit == 'wl':
                             vmin = np.abs(xmin*(dataset.redshift + 1) - l_ref)/l_ref * 299792.458
                         else:
                             vmin = np.abs(xmin)
-                    else:
-                        vmin = region.velspan
 
                     if xmax:
                         if xunit == 'wl':
                             vmax = np.abs(xmax*(dataset.redshift + 1) - l_ref)/l_ref * 299792.458
                         else:
                             vmax = np.abs(xmax)
-                    else:
-                        vmax = region.velspan
-                    plot_span = (vmax + vmin) / 2.
-                    include_velspan = plot_span * 0.8
-                    if np.abs(delta_v) <= include_velspan:
+
+                    # plot_span = (vmax + vmin) / 2.
+                    # include_velspan = plot_span * 0.8
+                    if vmin*0.8 <= delta_v <= vmax*0.8:
                         included_lines.append(line.tag)
 
     # --- Pack keyword arguments for plot_single_line:
@@ -461,11 +466,11 @@ def plot_all_lines(dataset, plot_fit=True, rebin=1, fontsize=12, xmin=None,
                 # num += 1
                 # LIV is a shorthand for 'lines_in_view'
 
+        fig.set_tight_layout(True)
         if filename:
             pdf.savefig(fig)
 
     if len(pages) > 0:
-        fig.set_tight_layout(True)
         if filename:
             pdf.close()
             print(" - Saved the fitted lines to PDF file: %s" % filename)
@@ -714,19 +719,24 @@ def plot_single_line(dataset, line_tag, index=0, plot_fit=False,
         ax = fig.add_subplot(111)
         fig.subplots_adjust(bottom=0.15, right=0.97, top=0.98)
 
+    if hasattr(region.velspan, '__iter__'):
+        vmin, vmax = region.velspan
+    else:
+        vspan = region.velspan
+        vmin = -vspan
+        vmax = vspan
+
     if not xmin:
         if xunit == 'vel':
-            xmin = -region.velspan
+            xmin = vmin
         else:
             xmin = x.min()
     if not xmax:
         if xunit == 'vel':
-            xmax = region.velspan
+            xmax = vmax
         else:
             xmax = x.max()
     ax.set_xlim(xmin, xmax)
-    # if np.abs(xmin) > 900 or np.abs(xmax) > 900:
-    #     ax.ticklabel_format(style='sci', axis='x', scilimits=(0, 0))
 
     if plot_fit and (isinstance(dataset.best_fit, dict) or
                      isinstance(dataset.pars, dict)):
@@ -1103,11 +1113,18 @@ def plot_residual(dataset, line_tag, index=0, rebin=1, xmin=None, xmax=None, axi
     vel = (x - l_ref) / l_ref * 299792.458
     y = y - profile
 
+    if hasattr(region.velspan, '__iter__'):
+        vmin, vmax = region.velspan
+    else:
+        vspan = region.velspan
+        vmin = -vspan
+        vmax = vspan
+
     if not xmin:
-        xmin = -region.velspan
+        xmin = vmin
 
     if not xmax:
-        xmax = region.velspan
+        xmax = vmax
     ax.set_xlim(xmin, xmax)
 
     view_part = (vel > xmin) * (vel < xmax)
@@ -1714,6 +1731,7 @@ def print_metallicity(dataset, params, logNHI, err=0.1):
     print("")
 
 
+logN_matcher = re.compile('logN[0-9]+_[A-Z][A-Z]?[0-9]?[a-z]?[I-Z]+[0-9]?[a-z]?')
 def print_total(dataset, verbose=True):
     """
     Print the total column densities of all species. This will sum *all*
@@ -1732,11 +1750,14 @@ def print_total(dataset, verbose=True):
             logN_err = []
             N_tot = []
             for par in params.keys():
-                if par.find('logN') >= 0 and par.split('_')[1] == ion:
-                    N_tot.append(params[par].value)
-                    if params[par].stderr < 1.0:
-                        logN.append(params[par].value)
-                        logN_err.append(params[par].stderr)
+                is_logN_par = logN_matcher.fullmatch(par)
+                if is_logN_par is not None:
+                    par_id, par_ion = par.split('_')
+                    if par_ion == ion:
+                        N_tot.append(params[par].value)
+                        if params[par].stderr < 1.0:
+                            logN.append(params[par].value)
+                            logN_err.append(params[par].stderr)
 
             if len(logN) > 0:
                 ION = [np.random.normal(n, e, 10000) for n, e in zip(logN, logN_err)]
@@ -2068,7 +2089,6 @@ def save_individual_components(dataset, filename, path=''):
 
     if dataset.best_fit:
         pars = dataset.best_fit
-        sampling = 3
         # Calculate profiles for each component of each ion:
         component_list = list()
         component_header = list()
@@ -2080,20 +2100,19 @@ def save_individual_components(dataset, filename, path=''):
                 component_profile = list()
                 for chunk in dataset.data:
                     x = chunk['wl']
-                    dx = np.mean(np.diff(x))
-                    xmin = np.log10(x.min() - 50*dx)
-                    xmax = np.log10(x.max() + 50*dx)
-                    N = sampling * len(x)
-                    profile_wl = np.logspace(xmin, xmax, N)
-                    tau = np.zeros_like(profile_wl)
-                    # Calculate actual pixel size in km/s:
-                    pxs = np.diff(profile_wl)[0] / profile_wl[0] * 299792.458
-                    # Set Gaussian kernel width:
                     if isinstance(chunk['res'], float):
+                        dx = np.mean(np.diff(x))
+                        xmin = np.log10(x.min() - 50*dx)
+                        xmax = np.log10(x.max() + 50*dx)
+                        sampling = 3
+                        N = sampling * len(x)
+                        profile_wl = np.logspace(xmin, xmax, N)
+                        pxs = np.diff(profile_wl)[0] / profile_wl[0] * 299792.458
                         kernel = chunk['res'] / pxs / 2.35482
                     else:
-                        print(" LSF kernel is not supported yet... skipping this data chunk!")
-                        continue
+                        kernel = load_lsf(chunk['res'], x, chunk['nsub'])
+                        profile_wl = np.linspace(x.min(), x.max(), chunk['nsub']*len(x))
+                    tau = np.zeros_like(profile_wl)
 
                     for line in lines:
                         l0, f, gam = line.get_properties()
@@ -2112,13 +2131,8 @@ def save_individual_components(dataset, filename, path=''):
                         profile_obs = np.interp(x, profile_wl, profile_broad)
 
                     else:
-                        print(" LSF kernel is not supported yet... skipping this data chunk!")
-                        # profile_broad = convolve_numba(profile, kernel)
-                        # if kernel_nsub > 1:
-                        #     # Interpolate onto the data grid:
-                        #     profile_obs = np.interp(x, profile_wl, profile_broad)
-                        # else:
-                        #     profile_obs = profile_broad
+                        profile_broad = voigt.convolve_numba(profile, kernel)
+                        profile_obs = np.interp(x, profile_wl, profile_broad)
                     component_wl.append(x)
                     component_profile.append(profile_obs)
 
