@@ -80,8 +80,13 @@ def main():
     if args.version:
         return
 
-    parameters = io.parse_input.parse_parameters(parfile)
     print(" Reading Parameters from file: " + parfile)
+    try:
+        parameters = io.parse_input.parse_parameters(parfile)
+    except io.parse_input.InputParserError as err_msg:
+        print("An error happened during the parsing of the input file:")
+        print(err_msg)
+        return
 
     name = parameters['name']
     # -- Load DataSet if the file already exists
@@ -116,6 +121,7 @@ def main():
             print(list(dataset.lines.keys()))
             print(" - Lines in parameter file:")
             print(parameters['lines'])
+
         for tag, velspan in parameters['lines']:
             if tag not in dataset.all_lines:
                 new_lines.append([tag, velspan])
@@ -191,7 +197,7 @@ def main():
                     if velspan is None:
                         velspan = dataset.velspan
                     for reg in regions_of_line:
-                        if np.abs(reg.velspan - velspan) < 0.1:
+                        if reg.velspan != velspan:
                             if verbose:
                                 print(" Detected difference in velocity span: %s" % ground_state)
                             dataset.verbose = False
@@ -248,7 +254,7 @@ def main():
         dataset = container.dataset.DataSet(parameters['z_sys'], parameters['name'])
 
         if 'velspan' in parameters.keys():
-            dataset.velspan = parameters['velspan']
+            dataset.set_velspan(parameters['velspan'])
 
         # Load data:
         for fname, res, norm, airORvac, nsub, ext, use_mask in parameters['data']:
